import os

from apiflask import APIBlueprint
from flask import jsonify

from db import db
from helper_functions import get_user_id_from_username, get_username_or_abort_401
from models import Transaction
from auth import auth

portfolio_blueprint = APIBlueprint('portfolio', __name__, url_prefix='/api')
__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))


@portfolio_blueprint.route('/portfolio', methods=['GET'])
@portfolio_blueprint.output(200)
@portfolio_blueprint.auth_required(auth)
@portfolio_blueprint.doc(summary="Returns portfolio", description="Returns all shares of current user")
def get_portfolio():
    username = get_username_or_abort_401()

    return_portfolio = {}
    transactions = db.session.query(Transaction).filter_by(user_id=get_user_id_from_username(username)).all()

    if transactions is not None:
        for row in transactions:
            if row.symbol in return_portfolio:
                return_portfolio[row.symbol]['count'] += row.count
                return_portfolio[row.symbol]['last_transaction'] = row.time
            else:
                return_portfolio[row.symbol] = {"count": row.count, "last_transaction": row.time}

    return jsonify({"status": 200, "text": "Successfully loaded symbols", "data": return_portfolio})
